package jMSMessageLogParser;

import java.util.ArrayList;
import java.util.Hashtable;
import java.util.StringTokenizer;

import org.eclipse.hyades.logging.events.cbe.CommonBaseEvent;
import org.eclipse.hyades.logging.events.cbe.ComponentIdentification;
import org.eclipse.hyades.logging.events.cbe.ReportSituation;
import org.eclipse.hyades.logging.events.cbe.Situation;
import org.eclipse.hyades.logging.parsers.LogParserException;
import org.eclipse.hyades.logging.parsers.Parser;

/**
 * JMS Message Log Parser class.
 * 
 * @author masuhara
 */
public class JMSMessageLogParser extends Parser {

	/* M */
	private final String JMS_RECEIVE = "---------Receive----------";

	/* M */
	private final String JMS_SEND = "---------Send----------";

	/* ʊ{CxgɃZbgׂ̎ */
	private String currentCreationTime;

	/* ÕOER[h̎Ԃێ */
	private String previousCreationTime = "";

	/* dx */
	private String severity;

	/* V[PXԍ */
	private int messageCounter = 0;

	/**
	 * Oʊ{Cxgɕϊ܂B
	 */
	public CommonBaseEvent[] parseNext() throws LogParserException {
		CommonBaseEvent[] temp = null;

		arrayIndex = 0;

		try {
			// t@C̑SĂ̍sԗ
			while ((curLine = readALine()) != null) {
				curLine = curLine.trim();

				// currentCreationTime̎擾iSR[hʁj
				if (isRecordStart()) {
					commonProcess4AllRecords();
					continue;
				}

				// ϊΏۂ̃R[hǂ𔻒肷
				if (isTargetRecord()) {
					// ϊΏۃR[hpʏ
					commonProcess4TargetRecords();

					// bZ[W̐ݒ
					setupMsg();
					curLine = readALine().trim();

					// wb_̒l̐ݒ
					if (isHeaderStart()) {
						curLine = readALine().trim();
						setupFromHeader();
					} else {
						throw new LogParserException("wb_܂BԁF"
								+ currentCreationTime);
					}
					curLine = readALine().trim();

					// {fB̒l̐ݒ
					if (isBodyStart()) {
						curLine = readALine().trim();
						setupFromBody();
					} else {
						throw new LogParserException("{fB܂BԁF"
								+ currentCreationTime);
					}

					// creationTime̐ݒ
					messages[arrayIndex].setCreationTime(currentCreationTime);

					// dx̐ݒ
					setupSeverity();

					// V[PXԍ̐ݒ
					setupSequenceNumber();

					arrayIndex++;
					recordCount++;
				}
			}

			if (arrayIndex == 0) {
				temp = null;
			} else {
				for (int i = arrayIndex; i < MessageArraySize; i++) {
					messages[i] = null;
				}
				temp = messages;
			}

		} catch (Throwable throwable) {
			throw new LogParserException("ϊsF" + throwable.getMessage());
		}

		// R[h݂Ȃꍇ
		if (recordCount == 0) {
			throw new LogParserException("R[h݂܂B");
		}

		return temp;
	}
	
	public String getName() {
		return "JMSMessageLogParser";
	}

	public String getVersion() {
		return "V1.0";
	}
	
	/**
	 * SR[hʂ̏B Ԃdx擾ăCX^XϐɃZbgB
	 */
	private void commonProcess4AllRecords() {
		StringTokenizer token = new StringTokenizer(curLine, " ");
		String tempTime = token.nextToken() + " " + token.nextToken();
		StringBuffer sbf = new StringBuffer(tempTime);
		sbf.replace(10, 11, "T");
		sbf.replace(19, 23, ".000000");

		currentCreationTime = sbf.toString().trim();

		// dx̎擾
		if (token.nextToken().equals("[JmsSessionDispatcher:")) {
			token.nextToken();
		}
		severity = token.nextToken();
	}

	/**
	 * ϊΏۃR[h̋ʏs܂B
	 */
	private void commonProcess4TargetRecords() {
		messages[arrayIndex].init();

		// ʊ{Cxg̃\[XR|[lgID
		ComponentIdentification sourceComponentID = eventFactory
				.createComponentIdentification();
		sourceComponentID.setExecutionEnvironment("Java");
		sourceComponentID.setComponent("JMSMessageLogParser");
		sourceComponentID.setSubComponent("JMSMessageLogParser");
		sourceComponentID.setComponentIdType("Application");
		sourceComponentID.setComponentType("JMSMessageLog");
		sourceComponentID.setLocation(localHostId);
		sourceComponentID.setLocationType(localHostIdFormat);
		messages[arrayIndex].setSourceComponentId(sourceComponentID);

		// ʊ{Cxg̃V`G[V
		Situation cbeSituation = eventFactory.createSituation();
		ReportSituation cbeReportSituation = eventFactory
				.createReportSituation();
		cbeReportSituation.setReasoningScope("INTERNAL");
		cbeReportSituation.setReportCategory("LOG");
		cbeSituation.setCategoryName("ReportSituation");
		cbeSituation.setSituationType(cbeReportSituation);
		messages[arrayIndex].setSituation(cbeSituation);
	}

	/**
	 * ݍsR[h̐擪ł邩𔻒肵܂B
	 * 
	 * @return ݍsR[h̐擪̏ꍇtrue^ȊOfalse
	 */
	private boolean isRecordStart() {
		return curLine.matches("^[1-2][0-9]{3}[-][0-1][0-9][-][0-3][0-9].*");
	}

	/**
	 * ݍsgpăR[hϊΏۂ𔻒肵܂B
	 * 
	 * @return R[hϊΏۂ̏ꍇtrue^ȊOfalse
	 */
	private boolean isTargetRecord() {
		return curLine.trim().equals(JMS_RECEIVE)
				|| curLine.trim().equals(JMS_SEND);
	}

	/**
	 * ݍswb_̎n܂肩ǂ肵܂B
	 * 
	 * @return ݍswb_̎n܂̏ꍇtrue^ȊOfalse
	 */
	private boolean isHeaderStart() {
		return curLine.trim().equals("Header{");
	}

	/**
	 * ݍswb_̏I肩ǂ𔻒肵܂B
	 * 
	 * @return ݍswb_̏Ȉꍇtrue^ȊOfalse
	 */
	private boolean isHeaderFinish() {
		return curLine.trim().equals("}");
	}

	/**
	 * ݍs{fB̎n܂肩ǂ肵܂B
	 * 
	 * @return ݍs{fB̎n܂̏ꍇtrue^ȊOfalse
	 */
	private boolean isBodyStart() {
		return curLine.trim().equals("Body {");
	}

	/**
	 * ݍs{fB̏I肩ǂ𔻒肵܂B
	 * 
	 * @return ݍs{fB̏Ȉꍇtrue^ȊOfalse
	 */
	private boolean isBodyFinish() {
		return curLine.trim().equals("}");
	}

	/**
	 * ʊ{CxgɃbZ[Wݒ肵܂B
	 */
	private void setupMsg() {
		if (curLine.trim().equals(JMS_RECEIVE)) {
			messages[arrayIndex].setMsg("Receive");
		} else {
			messages[arrayIndex].setMsg("Send");
		}
	}

	/**
	 * wb_l擾ċʊ{CxgɃZbg܂B
	 * 
	 * @throws LogParserException
	 */
	private void setupFromHeader() throws LogParserException {
		while (!isHeaderFinish()) {
			StringTokenizer token = new StringTokenizer(curLine, ":");
			String label = token.nextToken().trim();
			String value = null;
			if (label.equals("jmsProperties")) {
				curLine = readALine();
				while (curLine.indexOf(":") >= 0) {
					token = new StringTokenizer(curLine, ":");
					messages[arrayIndex].addExtendedDataElement(label + ":"
							+ token.nextToken().trim(), token.nextToken()
							.trim());
					curLine = readALine();
				}
			} else if (label.equals("jmsMessageID")
					|| label.equals("jmsCorrelationID")) {
				if (token.countTokens() == 3) {
					token.nextToken();
					value = token.nextToken().trim() + ":"
							+ token.nextToken().trim();
				} else {
					value = token.nextToken().trim();
				}
				messages[arrayIndex].addExtendedDataElement(label, value);
				curLine = readALine();
			} else {
				messages[arrayIndex].addExtendedDataElement(label, token
						.nextToken().trim());
				curLine = readALine();
			}
		}
	}

	/**
	 * {fBl擾ċʊ{CxgɃZbg܂B
	 * 
	 * @throws LogParserException
	 */
	private void setupFromBody() throws LogParserException {
		String bodyName = curLine;
		ArrayList bodyList = new ArrayList();
		curLine = readALine();
		while (!isBodyFinish()) {
			bodyList.add(curLine);
			curLine = readALine();
		}
		messages[arrayIndex].addExtendedDataElement(bodyName,
				(String[]) bodyList.toArray(new String[bodyList.size()]));
	}

	/**
	 * dx̒lݒ肵܂B
	 */
	private void setupSeverity() {
		if (severity.equals("INFO") || severity.equals("DEBUG"))
			messages[arrayIndex].setSeverity(((short) (10)));
		else if (severity.equals("Error"))
			messages[arrayIndex].setSeverity(((short) (50)));
		else if (severity.equals("Warning"))
			messages[arrayIndex].setSeverity(((short) (30)));
	}

	/**
	 * ÕR[hcreationTimeꍇ̓V[PXԍU܂B
	 */
	private void setupSequenceNumber() {
		if (previousCreationTime.equals(currentCreationTime)) {
			messages[arrayIndex]
					.setSequenceNumber(((short) (messageCounter++)));
		} else {
			messageCounter = 0;
			previousCreationTime = currentCreationTime;
		}
	}
}